==> star-engine-0.8.0.src.zip: star-engine-0.8.0.src/StarEngine.Core/AffineMatrix.cs <==
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace StarEngine.Core
{
    /// <summary>
    /// AtBϊ̂߂̍słB
    /// </summary>
    public struct AffineMatrix
    {
        /// <summary>
        /// sB
        /// </summary>
        public static AffineMatrix O
        {
            get { return new AffineMatrix(0, 0, 0, 0, 0, 0); }
        }

        /// <summary>
        /// PʍsB
        /// </summary>
        public static AffineMatrix I
        {
            get { return new AffineMatrix(1, 0, 0, 1, 0, 0); }
        }

        public static bool operator ==(AffineMatrix m1, AffineMatrix m2)
        {
            return m1.A == m2.A && m1.B == m2.B && m1.C == m2.C && m1.D == m2.D &&
                m1.TX == m2.TX && m1.TY == m2.TY;
        }

        public static bool operator !=(AffineMatrix m1, AffineMatrix m2)
        {
            return !(m1 == m2);
        }

        public override bool Equals(object obj)
        {
            return obj is AffineMatrix && this == (AffineMatrix)obj;
        }

        public override int GetHashCode()
        {
            return this.A.GetHashCode() ^ this.B.GetHashCode() ^
                this.C.GetHashCode() ^ this.D.GetHashCode() ^
                this.TX.GetHashCode() ^ this.TY.GetHashCode();
        }

        public override string ToString()
        {
            return string.Format("[[{0},{1},{4}],[{2},{3},{5}],[0,0,1]]",
                this.A, this.B, this.C, this.D, this.TX, this.TY);
        }

        public double A;
        public double B;
        public double C;
        public double D;
        public double TX;
        public double TY;

        public AffineMatrix(double a, double b, double c, double d, double tx, double ty)
        {
            this = new AffineMatrix();
            this.A = a;
            this.B = b;
            this.C = c;
            this.D = d;
            this.TX = tx;
            this.TY = ty;
        }

        /// <summary>
        /// ۂl擾܂B
        /// </summary>
        public bool IsRegular
        {
            get
            {
                return this.A * this.D - this.B * this.C != 0;
            }
        }

        /// <summary>
        /// AtBϊ܂B
        /// </summary>
        /// <param name="m">AtBϊsB</param>
        /// <returns>̍sB</returns>
        public AffineMatrix Concat(AffineMatrix m)
        {
            return this = new AffineMatrix(
                m.A * this.A + m.B * this.C,
                m.A * this.B + m.B * this.D,
                m.C * this.A + m.D * this.C,
                m.C * this.B + m.D * this.D,
                m.TX + m.A * this.TX + m.B * this.TY,
                m.TY + m.C * this.TX + m.D * this.TY);
        }

        /// <summary>
        /// tϊɂ܂B
        /// </summary>
        /// <returns>̍sB</returns>
        public AffineMatrix Invert()
        {
            double det = this.A * this.D - this.B * this.C;
            if (det == 0)
                throw new StarEngineException("not regular exception");
            double newA = this.D / det;
            double newB = -this.B / det;
            double newC = -this.C / det;
            double newD = this.A / det;

            return this = new AffineMatrix(
                newA, newB, newC, newD,
                newA * -this.TX + newB * -this.TY,
                newC * -this.TX + newD * -this.TY);
        }

        /// <summary>
        /// _ɑ΂ĕϊs܂B
        /// </summary>
        /// <param name="point">ϊO̍WB</param>
        /// <returns>ϊ̍WB</returns>
        public PointF Transform(PointF point)
        {
            return new PointF(
                (float)(this.A * point.X + this.B * point.Y + this.TX),
                (float)(this.C * point.X + this.D * point.Y + this.TY)
            );
        }
    }
}

==> star-engine-0.8.2.src.zip: star-engine-0.8.2.src/StarEngine.Core/AffineMatrix.cs <==
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace StarEngine.Core
{
    /// <summary>
    /// AtBϊ̂߂̍słB
    /// </summary>
    public struct AffineMatrix
    {
        /// <summary>
        /// s擾܂B
        /// </summary>
        public static AffineMatrix O
        {
            get { return new AffineMatrix(0, 0, 0, 0, 0, 0); }
        }

        /// <summary>
        /// Pʍs擾܂B
        /// </summary>
        public static AffineMatrix I
        {
            get { return new AffineMatrix(1, 0, 0, 1, 0, 0); }
        }

        /// <summary>
        /// 2  AffineMatrix lǂ𔻒f܂B
        /// </summary>
        /// <param name="m1">AffineMatrix IuWFNgB</param>
        /// <param name="m2">AffineMatrix IuWFNgB</param>
        /// <returns>m1  m2 l̏ꍇ trueBȊȌꍇ falseB</returns>
        public static bool operator ==(AffineMatrix m1, AffineMatrix m2)
        {
            return m1.A == m2.A && m1.B == m2.B && m1.C == m2.C && m1.D == m2.D &&
                m1.TX == m2.TX && m1.TY == m2.TY;
        }

        /// <summary>
        /// 2  AffineMatrix قȂ邩ǂ𔻒f܂B
        /// </summary>
        /// <param name="m1">AffineMatrix IuWFNgB</param>
        /// <param name="m2">AffineMatrix IuWFNgB</param>
        /// <returns>m1  m2 قȂꍇ trueBȊȌꍇ falseB</returns>
        public static bool operator !=(AffineMatrix m1, AffineMatrix m2)
        {
            return !(m1 == m2);
        }

        /// <summary>
        /// ̃IuWFNglǂ𔻒f܂B
        /// </summary>
        /// <param name="obj">IuWFNgB</param>
        /// <returns>̃CX^X obj l̏ꍇ trueBȊȌꍇ falseB</returns>
        public override bool Equals(object obj)
        {
            return obj is AffineMatrix && this == (AffineMatrix)obj;
        }

        /// <summary>
        ///  AffineMatrix ̃nbVR[hԂ܂B
        /// </summary>
        /// <returns>32 rbgtnbVR[hB</returns>
        public override int GetHashCode()
        {
            return this.A.GetHashCode() ^ this.B.GetHashCode() ^
                this.C.GetHashCode() ^ this.D.GetHashCode() ^
                this.TX.GetHashCode() ^ this.TY.GetHashCode();
        }

        /// <summary>
        ///  AffineMatrix ̕\Ԃ܂B
        /// </summary>
        /// <returns>AffineMatrix ̕\B</returns>
        public override string ToString()
        {
            return string.Format("[[{0},{1},{4}],[{2},{3},{5}],[0,0,1]]",
                this.A, this.B, this.C, this.D, this.TX, this.TY);
        }

        /// <summary>
        /// A B
        /// </summary>
        public double A;

        /// <summary>
        /// B B
        /// </summary>
        public double B;

        /// <summary>
        /// C B
        /// </summary>
        public double C;

        /// <summary>
        /// D B
        /// </summary>
        public double D;

        /// <summary>
        /// X ̕sړB
        /// </summary>
        public double TX;

        /// <summary>
        /// Y ̕sړB
        /// </summary>
        public double TY;

        /// <summary>
        /// V AffineMatrix IuWFNg܂B
        /// </summary>
        /// <param name="a">A B</param>
        /// <param name="b">B B</param>
        /// <param name="c">C B</param>
        /// <param name="d">D B</param>
        /// <param name="tx">X ̕sړB</param>
        /// <param name="ty">Y ̕sړB</param>
        public AffineMatrix(double a, double b, double c, double d, double tx, double ty)
        {
            this = new AffineMatrix();
            this.A = a;
            this.B = b;
            this.C = c;
            this.D = d;
            this.TX = tx;
            this.TY = ty;
        }

        /// <summary>
        /// ۂl擾܂B
        /// </summary>
        public bool IsRegular
        {
            get
            {
                return this.A * this.D - this.B * this.C != 0;
            }
        }

        /// <summary>
        /// AtBϊ܂B
        /// </summary>
        /// <param name="m">AtBϊsB</param>
        /// <returns>̍sB</returns>
        public AffineMatrix Concat(AffineMatrix m)
        {
            return this = new AffineMatrix(
                m.A * this.A + m.B * this.C,
                m.A * this.B + m.B * this.D,
                m.C * this.A + m.D * this.C,
                m.C * this.B + m.D * this.D,
                m.TX + m.A * this.TX + m.B * this.TY,
                m.TY + m.C * this.TX + m.D * this.TY);
        }

        /// <summary>
        /// tsɂ܂B
        /// </summary>
        /// <returns>̍sB</returns>
        public AffineMatrix Invert()
        {
            double det = this.A * this.D - this.B * this.C;
            if (det == 0)
                throw new StarEngineException("not regular exception");
            double newA = this.D / det;
            double newB = -this.B / det;
            double newC = -this.C / det;
            double newD = this.A / det;

            return this = new AffineMatrix(
                newA, newB, newC, newD,
                newA * -this.TX + newB * -this.TY,
                newC * -this.TX + newD * -this.TY);
        }

        /// <summary>
        /// _ɑ΂ĕϊs܂B
        /// </summary>
        /// <param name="point">ϊO̍WB</param>
        /// <returns>ϊ̍WB</returns>
        public PointF Transform(PointF point)
        {
            return new PointF(
                (float)(this.A * point.X + this.B * point.Y + this.TX),
                (float)(this.C * point.X + this.D * point.Y + this.TY)
            );
        }
    }
}
